/**************************************************************************************************

Copyright (c) 2014-2015   Tsunehiko Kato  and  4D2U Project, National Astronomical Observatory of Japan

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

ȉɒ߂ɏ]A{\tgEFAъ֘Ãt@Ciȉu\tgEFAvj̕擾
邷ׂĂ̐lɑ΂A\tgEFA𖳐ɈƂ𖳏ŋ܂Bɂ́A\tgEFA̕
gpAʁAύXAAfځAЕzATuCZXA/܂͔̔錠Aу\tgEFA
񋟂鑊ɓƂ錠Ɋ܂܂܂B

L̒쌠\і{\A\tgEFÂׂĂ̕܂͏dvȕɋLڂ̂Ƃ
B

\tgEFÁû܂܁vŁAł邩Öقł邩킸A̕ۏ؂Ȃ񋟂܂B
łۏ؂Ƃ́AiA̖ړIւ̓KAьNQɂĂ̕ۏ؂܂݂܂AɌ
̂ł͂܂B ҂܂͒쌠҂́A_sׁAs@sׁA܂͂ȊOł낤ƁA\t
gEFAɋN܂͊֘AA邢̓\tgEFA̎gp܂͂̑̈ɂĐ؂̐A
QȂ̋`ɂĉ̐ӔCȂ̂Ƃ܂B
***************************************************************************************************/
#include  <Windows.h>
#include  <tchar.h>
#include  <stdio.h>
#include  <atlbase.h>
#include  <dshow.h>
#include  "mtk_movie.h"
#include  "mtk_window_mode.h"




#define SAFE_RELEASE(x) { if (x) x->Release(); x = NULL; }


//****
extern bool	bNetwork;
extern int disp_w, disp_h;







mtk_movie::mtk_movie() :
	m_pGB(NULL),
	m_pMC(NULL),
	m_pEv(NULL),
	m_pMS(NULL),
	m_pVideo(NULL),
	m_state(STATE_NO_MOVIE),
	m_end_pos(0)
{

}


mtk_movie::~mtk_movie()
{
	CleanUpGraph();
}



void
mtk_movie::CleanUpGraph()
{
	if (m_pMC) {
		m_pMC->Stop();
	}

	SAFE_RELEASE(m_pMS);
	SAFE_RELEASE(m_pEv);
	SAFE_RELEASE(m_pMC);

	delete m_pVideo;
	m_pVideo = NULL;

	SAFE_RELEASE(m_pGB);

	m_state = STATE_NO_MOVIE;
	m_end_pos = 0;
}



HRESULT
mtk_movie::Open(const char *fn, HWND hwnd, int x0, int y0, int w, int h, UINT u_WM_NOTIFY, int iRenderer)
{
	HRESULT hr;
	WCHAR wFile[MAX_PATH];



	//--  Clear
	CleanUpGraph();


	//--  Init Graph
	hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void**)&m_pGB);
	if (FAILED(hr)) {
		goto done;
	}


	//--  Create Interfaces
	hr = m_pGB->QueryInterface(IID_IMediaControl, (void**)&m_pMC);
	if (FAILED(hr)) {
		goto done;
	}

	hr = m_pGB->QueryInterface(IID_IMediaEventEx, (void**)&m_pEv);
	if (FAILED(hr)) {
		goto done;
	}
	hr = m_pEv->SetNotifyWindow((OAHWND)hwnd, u_WM_NOTIFY, NULL);
	if (FAILED(hr)) {
		goto done;
	}

	hr = m_pGB->QueryInterface(IID_IMediaSeeking, (void**)&m_pMS);
	if (FAILED(hr)) {
		goto done;
	}



	//--  Create Video Renderer
	switch (iRenderer) {
	case RENDERER_WINDOWED_VMR7:
		m_pVideo = new (mtk_video_window_mode);
		break;
	case RENDERER_WINDOWLESS_VMR7:
		m_pVideo = new (mtk_video_windowless_VMR7);
		break;
	case RENDERER_WINDOWLESS_VMR9:
		m_pVideo = new (mtk_video_windowless_VMR9);
		break;
	case RENDERER_EVR:
		m_pVideo = new (mtk_video_EVR);
		break;
	}

	if (m_pVideo == NULL) {
		hr = E_OUTOFMEMORY;
		goto done;
	}



	
	
	hr = m_pVideo->Init(hwnd, m_pGB);
	if (FAILED(hr)) {
		/*
		char str[200];
		sprintf(str, "Init failed!  hr=%x", hr);
		MessageBox(hwnd, str, "Error", MB_OK);
		*/
		goto done;
	}
	
	


	//--  File Open
	wcscpy(wFile, CA2W(fn));
	hr = m_pGB->RenderFile(wFile, NULL);
	if (FAILED(hr)) {
		goto done;
	}




	//--  State
	m_state = STATE_STOPPED;
	hr = m_pMS->GetStopPosition(&m_end_pos);




	//--  Set View Window (Fit to Window)
	long lw, lh;
	m_pVideo->GetVideoSize(&lw, &lh) ;

	float rt_wn = float(h) / float(w);
	float rt_mv = float(lh) / float(lw);
	if (rt_mv > rt_wn) {
		// EBhE[r[̕c
		m_w = int(h / rt_mv);
		m_h = h;
	}
	else {
		// [r[EBhE̕c
		m_w = w;
		m_h = int(w*rt_mv);
	}
	
	int dx, dy;
	dx = (w - m_w) / 2;
	dy = (h - m_h) / 2;
	m_x = x0 + dx;
	m_y = y0 + dy;

	bool bFullScreen = (!bNetwork && is_fullscreen_mode()) || (bNetwork && (w==disp_w) && (h==disp_h));


	hr = m_pVideo->SetViewWindow(hwnd, m_x, m_y, m_w, m_h, bFullScreen);


	m_pMC->Pause();


	do {
		Sleep(10);
		SetForegroundWindow(hwnd);
	} while (GetForegroundWindow() != hwnd);


done:
	if (FAILED(hr)) {
		CleanUpGraph();
	}
	return hr;
}



void
mtk_movie::Play()
{
	if (m_state != STATE_PAUSED && m_state != STATE_STOPPED) {
		return;
	}

	HRESULT hr = m_pMC->Run();
	if (SUCCEEDED(hr)) {
		m_state = STATE_PLAYING;
	}
}



void
mtk_movie::Pause()
{
	if (m_state != STATE_PLAYING) {
		return;
	}

	HRESULT hr = m_pMC->Pause();
	if (SUCCEEDED(hr)) {
		m_state = STATE_PAUSED;
	}
}



void
mtk_movie::Home()
{
	SetPosition(0);
}



void
mtk_movie::End()
{
	SetPosition(m_end_pos);
}



void
mtk_movie::Close()
{
	CleanUpGraph();
}



LONGLONG
mtk_movie::GetPosition() const
{
	if (m_state == STATE_NO_MOVIE) {
		return 0;
	}

	LONGLONG pos;
	HRESULT hr = m_pMS->GetCurrentPosition(&pos);
	if (FAILED(hr)) {
		return 0;
	}
	return pos;
}


LONGLONG
mtk_movie::GetEndPosition() const
{
	if (m_state == STATE_NO_MOVIE) {
		return 0;
	}

	return m_end_pos;
}



void
mtk_movie::SetPosition(LONGLONG pos)
{
	if (m_state == STATE_NO_MOVIE) {
		return;
	}

	if (m_state != STATE_PAUSED) {
		HRESULT hr = m_pMC->Pause();
		if (FAILED(hr)) {
			return;
		}
		m_state = STATE_PAUSED;
	}

	m_pMS->SetPositions(&pos, AM_SEEKING_AbsolutePositioning | AM_SEEKING_SeekToKeyFrame, NULL, AM_SEEKING_NoPositioning);
}


void
mtk_movie::GetWindowPosition(int &x, int &y, int &w, int &h) const
{
	x = m_x;
	y = m_y;
	w = m_w;
	h = m_h;
}


int
mtk_movie::GetState() const
{
	return  m_state;
}



HRESULT
mtk_movie::OnPaint(HWND hwnd, HDC hdc)
{
	if (m_pVideo) {
		return m_pVideo->OnPaint(hwnd, hdc);
	}
	else {
		return S_OK;
	}
}



HRESULT
mtk_movie::OnDisplayChange()
{
	if (m_pVideo) {
		return m_pVideo->OnDisplayChange();
	}
	else {
		return S_OK;
	}
}



bool
mtk_movie::GetEvent(long& evCode, long& prm1, long& prm2)
{
	if (m_pEv == NULL)  return false;

	if (!SUCCEEDED(m_pEv->GetEvent(&evCode, &prm1, &prm2, 0)))	return false;
	if (!SUCCEEDED(m_pEv->FreeEventParams(evCode, prm1, prm2)))	return false;

	return true;
}





//---------------
//  Media  File
//---------------
bool
MediaFiles::open_info_file(const directory& dir, const char* fn)
{
	FILE* fp = dir.fopen(fn, "rt");
	if (fp == NULL)  return false;
	clear();


	char one_line[512];
	int count;


	//-- efBAt@C̏擾
	while (true) {
		media_file_info  mfi;
		char mfn[MAX_PATH + 1], mfn2[MAX_PATH + 1], type[20], menu_title[256], disp_title[256];

		if (fgets(one_line, 512, fp) == NULL)  break;
		count = sscanf(one_line, "%s %s %s %s %s", mfn, mfn2, type, menu_title, disp_title);
		if (count != 5)  continue;

		switch (type[0]) {
		case 'I':	case 'i':		mfi.type = MEDIATYPE_IMAGE;				break;
		case 'P':	case 'p':		mfi.type = MEDIATYPE_POPUP_IMAGE;		break;
		case 'M':	case 'm':		mfi.type = MEDIATYPE_MOVIE;				break;
		case 'A':	case 'a':		mfi.type = MEDIATYPE_AUDIO;				break;
		default:															continue;
		}

		strcpy(mfi.filename, dir.get_path(mfn));
		strcpy(mfi.filename2, dir.get_path(mfn2));
		strcpy(mfi.menu_title, menu_title);
		strcpy(mfi.disp_title, disp_title);
		push_back(mfi);

		if (size() > media_files_max) break;
	}

	fclose(fp);
	return  true;
}


bool
MediaFiles::load_pathfile(int idx, char file_path[][MAX_PATH], bool bSubtitlesMode)  const
{
	if (idx<0 || idx >= size())  return false;

	const char* filename = (!bSubtitlesMode) ? (*this)[idx].filename : (*this)[idx].filename2;

	FILE *fp;
	if ((fp = fopen(filename, "rt")) == NULL)   return false;

	int i = 0;
	while (true) {
		char ch;
		char* p = file_path[i];
		while ((ch = getc(fp)) != '\n' && ch != EOF) {
			*p++ = ch;
		}
		*p = '\0';
		i++;
		if (ch == EOF)  break;
	}
	fclose(fp);
	return  true;
}
