/**************************************************************************************************

Copyright (c) 2007-2015   Tsunehiko Kato  and  4D2U Project, National Astronomical Observatory of Japan

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

ȉɒ߂ɏ]A{\tgEFAъ֘Ãt@Ciȉu\tgEFAvj̕擾
邷ׂĂ̐lɑ΂A\tgEFA𖳐ɈƂ𖳏ŋ܂Bɂ́A\tgEFA̕
gpAʁAύXAAfځAЕzATuCZXA/܂͔̔錠Aу\tgEFA
񋟂鑊ɓƂ錠Ɋ܂܂܂B

L̒쌠\і{\A\tgEFÂׂĂ̕܂͏dvȕɋLڂ̂Ƃ
B

\tgEFÁû܂܁vŁAł邩Öقł邩킸A̕ۏ؂Ȃ񋟂܂B
łۏ؂Ƃ́AiA̖ړIւ̓KAьNQɂĂ̕ۏ؂܂݂܂AɌ
̂ł͂܂B ҂܂͒쌠҂́A_sׁAs@sׁA܂͂ȊOł낤ƁA\t
gEFAɋN܂͊֘AA邢̓\tgEFA̎gp܂͂̑̈ɂĐ؂̐A
QȂ̋`ɂĉ̐ӔCȂ̂Ƃ܂B
***************************************************************************************************/
#include  "mtk_planet_eclipse.h"
#include  "mitaka.h"
#include  "mtk_mode.h"
#include  "../common/astro/astro_const.h"
#include  "mtk_planet_position.h"
#include  "mtk_planet_orientation.h"
#include  "mtk_planet_phys_params.h"
#include  "mtk_satellite_data.h"
#include  "mtk_sun.h"
#include  "mtk_eclipse.h"
#include  "mtk_planetarium.h"
#include  "mtk_switches.h"



static float MoonEclipseR = 0.3;
static float MoonEclipseG = 0.0;

void
set_moon_eclipse_shadow_color(float R, float G)
{
	if (R < 0.0f) { R = 0.0f; }
	if (R > 1.0f) { R = 1.0f; }
	if (G < 0.0f) { G = 0.0f; }
	if (G > 1.0f) { G = 1.0f; }

	MoonEclipseR = R;
	MoonEclipseG = G;
}




//-------------
//  H̉e
//-------------
void
draw_planet_eclipse(const mtk_draw_struct& ds, int plnt_idx, int sat_idx)
{
	double R1, R2, Rs;
	vector3d r1, r2, rs;

	double Rx, Ry, Rz;
	get_satellite_size(ds.tyMode, sat_idx, false, Rx, Ry, Rz);


	r1 = get_planet_position(plnt_idx);
	R1 = get_planet_Re(ds.tyMode, plnt_idx) * _1km_pc;
	r2 = get_satellite_position(sat_idx);
	R2 = Rx * _1km_pc;
	rs = vector3d(0,0,0);
	Rs = get_sun_radius(ds.tyMode);


	matrix3d M = get_planet_ec_to_pl_matrix(plnt_idx);

	vector3d ex, ey, ez;
	get_planet_bases(plnt_idx, ex, ey, ez);

	float th_c = ds.rad_per_pixel;
	if (ds.tyMode == MODE_PLANETARIUM) {
		th_c /= get_planetarium_zooming_factor();
	}

	draw_eclipse_shadow(ds,
						r1, R1, ez, get_planet_f(plnt_idx),
						r2, R2, rs, Rs, M, th_c, 30, 30,
						(swEclipse & ECLIPSE_DISP), (swEclipse & ECLIPSE_BORDER),
						vector3f(0,0,0));
}



//---------------
//  qH̉e
//---------------
void
draw_satellite_eclipse(const mtk_draw_struct& ds, int sat_idx, int plnt_idx)
{
	double R1, R2, Rs;
	double Rx, Ry, Rz;
	vector3d r1, r2, rs;



	if (ds.tyMode == MODE_PLANETARIUM) {
		get_satellite_size(ds.tyMode, sat_idx, true, Rx, Ry, Rz);
		R2 = get_planet_real_Re(plnt_idx) * _1km_pc;
		Rs = R_sun_km * _1km_pc;
		if (sat_idx == SAT_MOON) {
			r1 = calc_lunar_real_position(ds.JD);
		}
		else {
			r1 = get_satellite_position(sat_idx);
		}
	}
	else {
		get_satellite_size(ds.tyMode, sat_idx, false, Rx, Ry, Rz);
		R2 = get_planet_Re(ds.tyMode, plnt_idx)*_1km_pc;
		Rs = get_sun_radius(ds.tyMode);
		r1 = get_satellite_position(sat_idx);
	}
	R1 = Rx * _1km_pc;
	r2 = get_planet_position(plnt_idx);
	rs = vector3d(0, 0, 0);

	matrix3d M = get_satellite_ec_to_sat_matrix(sat_idx);

	vector3d ex, ey, ez;
	get_satellite_bases(sat_idx, ex, ey, ez);

	float th_c = ds.rad_per_pixel;
	if (ds.tyMode == MODE_PLANETARIUM) {
		th_c /= get_planetarium_zooming_factor();
	}


	// {e̐F
	vector3f shd_col(0, 0, 0);
	if (sat_idx == SAT_MOON) {
		shd_col.set(MoonEclipseR, MoonEclipseG, 0.0);
	}


	draw_eclipse_shadow(ds,
						r1, R1, ez, 0.0,
						r2, R2, rs, Rs, M, th_c, 30, 30,
						(swEclipse & ECLIPSE_DISP), (swEclipse & ECLIPSE_BORDER),
						shd_col
						);
}
